{-# LANGUAGE PatternSynonyms, ScopedTypeVariables, ViewPatterns #-}


{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (inaki@blueleaf.cc)
-}

#define ENABLE_OVERLOADING (MIN_VERSION_haskell_gi_overloading(1,0,0) \
       && !defined(__HADDOCK_VERSION__))

module GI.Pango.Constants
    ( 
    pattern VERSION_MIN_REQUIRED            ,
    pattern UNKNOWN_GLYPH_WIDTH             ,
    pattern UNKNOWN_GLYPH_HEIGHT            ,
    pattern SCALE                           ,
    pattern RENDER_TYPE_NONE                ,
    pattern GLYPH_UNKNOWN_FLAG              ,
    pattern GLYPH_INVALID_INPUT             ,
    pattern GLYPH_EMPTY                     ,
    pattern ENGINE_TYPE_SHAPE               ,
    pattern ENGINE_TYPE_LANG                ,
    pattern ATTR_INDEX_FROM_TEXT_BEGINNING  ,
    pattern ANALYSIS_FLAG_IS_ELLIPSIS       ,
    pattern ANALYSIS_FLAG_CENTERED_BASELINE ,

    ) where

import Data.GI.Base.ShortPrelude
import qualified Data.GI.Base.ShortPrelude as SP
import qualified Data.GI.Base.Overloading as O
import qualified Prelude as P

import qualified Data.GI.Base.Attributes as GI.Attributes
import qualified Data.GI.Base.ManagedPtr as B.ManagedPtr
import qualified Data.GI.Base.GClosure as B.GClosure
import qualified Data.GI.Base.GError as B.GError
import qualified Data.GI.Base.GVariant as B.GVariant
import qualified Data.GI.Base.GValue as B.GValue
import qualified Data.GI.Base.GParamSpec as B.GParamSpec
import qualified Data.GI.Base.CallStack as B.CallStack
import qualified Data.GI.Base.Properties as B.Properties
import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map
import qualified Foreign.Ptr as FP
import qualified GHC.OverloadedLabels as OL


{- |
A macro that should be defined by the user prior to including
the pango.h header.
The definition should be one of the predefined Pango version
macros: @/PANGO_VERSION_1_2/@, @/PANGO_VERSION_1_4/@,...

This macro defines the earliest version of Pango that the package is
required to be able to compile against.

If the compiler is configured to warn about the use of deprecated
functions, then using functions that were deprecated in version
'GI.Pango.Constants.VERSION_MIN_REQUIRED' or earlier will cause warnings (but
using functions deprecated in later releases will not).

/Since: 1.42/
-}
pattern VERSION_MIN_REQUIRED = 2 :: Int32

{- |
/No description available in the introspection data./
-}
pattern UNKNOWN_GLYPH_WIDTH = 10 :: Int32

{- |
/No description available in the introspection data./
-}
pattern UNKNOWN_GLYPH_HEIGHT = 14 :: Int32

{- |
The 'GI.Pango.Constants.SCALE' macro represents the scale between dimensions used
for Pango distances and device units. (The definition of device
units is dependent on the output device; it will typically be pixels
for a screen, and points for a printer.) 'GI.Pango.Constants.SCALE' is currently
1024, but this may be changed in the future.

When setting font sizes, device units are always considered to be
points (as in \"12 point font\"), rather than pixels.
-}
pattern SCALE = 1024 :: Int32

{-# DEPRECATED RENDER_TYPE_NONE ["(Since version 1.38)"] #-}
{- |
A string constant defining the render type
for engines that are not rendering-system specific.
-}
pattern RENDER_TYPE_NONE = "PangoRenderNone" :: T.Text

{- |
The 'GI.Pango.Constants.GLYPH_UNKNOWN_FLAG' macro is a flag value that can be added to
a @/gunichar/@ value of a valid Unicode character, to produce a @/PangoGlyph/@
value, representing an unknown-character glyph for the respective @/gunichar/@.
-}
pattern GLYPH_UNKNOWN_FLAG = 268435456 :: Word32

{- |
The 'GI.Pango.Constants.GLYPH_INVALID_INPUT' macro represents a @/PangoGlyph/@ value that has a
special meaning of invalid input.  'GI.Pango.Objects.Layout.Layout' produces one such glyph
per invalid input UTF-8 byte and such a glyph is rendered as a crossed
box.

Note that this value is defined such that it has the 'GI.Pango.Constants.GLYPH_UNKNOWN_FLAG'
on.

/Since: 1.20/
-}
pattern GLYPH_INVALID_INPUT = 4294967295 :: Word32

{- |
The 'GI.Pango.Constants.GLYPH_EMPTY' macro represents a @/PangoGlyph/@ value that has a
 special meaning, which is a zero-width empty glyph.  This is useful for
example in shaper modules, to use as the glyph for various zero-width
Unicode characters (those passing 'GI.Pango.Functions.isZeroWidth').
-}
pattern GLYPH_EMPTY = 268435455 :: Word32

{-# DEPRECATED ENGINE_TYPE_SHAPE ["(Since version 1.38)"] #-}
{- |
A string constant defining the engine type for shaping engines.
These engines derive from 'GI.Pango.Objects.EngineShape.EngineShape'.
-}
pattern ENGINE_TYPE_SHAPE = "PangoEngineShape" :: T.Text

{-# DEPRECATED ENGINE_TYPE_LANG ["(Since version 1.38)"] #-}
{- |
A string constant defining the engine type for language engines.
These engines derive from 'GI.Pango.Objects.EngineLang.EngineLang'.
-}
pattern ENGINE_TYPE_LANG = "PangoEngineLang" :: T.Text

{- |
This value can be used to set the start_index member of a 'GI.Pango.Structs.Attribute.Attribute'
such that the attribute covers from the beginning of the text.

/Since: 1.24/
-}
pattern ATTR_INDEX_FROM_TEXT_BEGINNING = 0 :: Int32

{- |
This flag is used to mark runs that hold ellipsized text,
in an ellipsized layout.

/Since: 1.36.7/
-}
pattern ANALYSIS_FLAG_IS_ELLIPSIS = 2 :: Int32

{- |
Whether the segment should be shifted to center around the baseline.
Used in vertical writing directions mostly.

/Since: 1.16/
-}
pattern ANALYSIS_FLAG_CENTERED_BASELINE = 1 :: Int32


